# Systme de template
# date : 27.04.05
# auteur : G.Burri

class Template

   # p1 [String] : repertoire dans lequel se trouve le template.
   def initialize(rep)
      @rep = rep
      
      #les fichiers de template :
      #{<nom fichier> => [<date modification>, <contenu>,
      #  {<balise> => [{<var> => <value>}, <contenu>]}
      #]}
      @files = Hash::new
   end

   # Renvoie le contenu d'une variable
   # p1 : le fichier
   # p2 : le nom de la balise
   # p3 : le nom de la variable
   # ret : la valeur de la variable
   def getVar(file, markup, variable)
      cPath = getCompletePath(file)
      if @files[cPath][2].key?(markup)
         return @files[cPath][2][markup][0][variable]
      end
   end
   
   # Dfinie le contenu d'une balise
   # p1 : le fichier
   # p2 : la balise
   # p3 : le contenu
   def setMarkup(file, markup, content)   
      cPath = getCompletePath(file)
      load(cPath)
      if @files[cPath][2].key?(markup)
         @files[cPath][2][markup][1] = content
      else
         @files[cPath][2][markup] = [Hash::new, content]
      end
   end
   
   # Renvoie la page aprs remplacement des balises
   # par leur contenu
   def process(file)
      cPath = getCompletePath(file)
      load(cPath)
      content = @files[cPath][1].dup
      @files[cPath][2].each{|markup, markupContent|
         markupContent = markupContent[1]
         content.sub!('{' + markup + '}', markupContent)
      }
      return content
   end

private

   #retourne le chemin complet + le fichier
   def getCompletePath(file)
      return @rep + '/' + file
   end

   # Charge un fichier de template.
   # p1 [String] : le nom du fichier
   def load(file)
      content =  @files[file]
      #le fichier n'existe pas en cache ou sa date 
      #n'est pas la meme que celle en cache alors recharge le fichier      
      if content == nil or File::mtime(file) != content[0]
         f = Template::openFile(file)
         @files[file] = [File::mtime(file), f.read, Hash::new]
         f.close   
         
         #parse les balises et mmorise leurs variables
         @files[file][1].gsub!(/\{[^\}]*\}/){|m|
            vars = [Hash::new, m.dup]
            m.gsub!(/\s*([a-zA-Z0-9]+)\s*\=\s*\"([^\"]*)\"/m){|v|
               vars[0][$1] = $2
               '' #vire les parametres
            }
            @files[file][2][m[1..-2]] = vars
            m
         }
      end
   end
   
   def Template::openFile(file, mode='r')
      if /.*mswin32.*/ =~ RUBY_PLATFORM
         return File::new(file, mode + 'b')
      else
         return File::new(file, mode)
      end
   end   
end
