#!/usr/bin/ruby

# EliseWebServer
# Fichier : WebConnection.rb
# version : 1
# Date cration : 16.09.03
# Date modification : 16.09.03
#
# Connection TCP entre le serveur et un client.

require 'src/DirList.rb'
require 'src/Content.rb'
require 'src/InHttpMessage.rb'

require 'src/constants.rb'
require 'user_settings.rb'

$crlf = 13.chr + 10.chr #<cr><lf>

class WebConnection

   @@nbThread = 0 #le nombre de thread actif
   #pour limiter le nombre de thread

   def WebConnection::getNbThread
      return @@nbThread
   end

   def initialize(con)
      #si ya trop  de connexion alors renvois une erreur au client
      if @@nbThread >= MAX_THREADS
         begin
            ContentError::new(503).put(con)
            con.flush
            con.shutdown
         rescue Exception => e
         end
         return
      end
      t = Thread::start(con){|con|
         begin
            @@nbThread += 1

            #le nombre de message recut durant une connexion
            nbMess = 0

            while nbMess < MAX_NUMBER_OF_HTTP_MESSAGE_ON_THE_SAME_CONNEXION
               #coupe la connection si le client ne rpond pas aprs un certain temps
               break if IO::select([con], nil, nil, MAX_CONNEXION_WAIT) == nil
               client_data = con.recv(2048)
               nbMess += 1

               #FIXME !!!!! :
               #mthode de brute : tant qu'il n'y a pas deux \r\n de suite alors attend de nouvelles donnes
               n = 0
               while (/\r\n\r\n/ =~ client_data) == nil and n < 10
                  n += 1
                  break if IO::select([con], nil, nil, MAX_CONNEXION_WAIT) == nil
                  tmp = con.recv(2048)
                  client_data += tmp #[0]
               end
               #END FIXME !!!!!

               messageClient = InHttpMessage::new(client_data, con)
               
               #vrifie les droits d'accs
               right = ACCESS_RIGHTS[messageClient.getRelativeDirectory]
                              
               if right != nil
                  if messageClient.getFile == nil
                     if right.include?('NOLIST')
                        ContentError::new(403).put(con)
                        break
                     end
                  else
                     if right.include?('NOREAD')
                        ContentError::new(403).put(con)
                        break
                     end
                  end
               end

               content = Content::new(messageClient)
               content.keepAlive(messageClient.getOption('Connection') == 'keep-alive')
               content.setNbMess(nbMess)

               #si les dates sont les mme : le client  dj la bonne version => renvoie un 304
               if messageClient.getOption('If-Modified-Since') == content.getLastModifiedDate.getHTTP
                  ContentGenericCode::new(304).put(con)
               else #sinon renvoie le contenu
                  content.put(con)
               end

               #on coupe la connexion si ncessaire
               break if !content.keepAlive? or con.closed?

               con.flush
            end

         rescue  Exception => e
            puts e.message
            puts e.backtrace
         ensure
            @@nbThread -= 1
            con.shutdown
         end
      }
   end

end
