#!/usr/bin/ruby

# RubyWebServer
# Fichier : Content.rb
# version : 2
# Date cration : 21.06.04
# Date modification : 19.01.05
#
# Description :
# gestion du contenu d'une URL.


require 'DirList'
require 'thread'

require 'constants.rb'
require 'user_settings.rb'

#Renvois une instance d'une des classes suivante :
# - 'ContentRedirection' : redirection 
# - 'ContentFile' : le contenu d'un fichier
# - 'ContentDir' : le listage des fichiers d'un repertoire
# - 'ContentDynamic' : interprtation d'un fichier

class Content

   def Content::new(dir)
      #remplace les entits HTML, par exemple : '%20' -> ''
      HTML_ENTITY.each{|key, val|
         dir.gsub!(key, val)
      }
      
      if File::directory?(dir)
      
         #si il n'y a pas de /  la fin alors envois une redirection au client
         if (/^.*\/$/ =~ dir) == nil
             return ContentRedirection::new(dir+'/')     
         end

         #test si le repertoire contient un index par dfaut
         #si oui alors renvoie son contenu sinon renvoie la liste des fichiers
         index = Dir::entries(dir) & INDEXES
         if index.empty?
            return ContentDir::new(dir)
         else
            dir += index[0]
            return Content::contentFile(dir)
         end
         
      end
      return Content::contentFile(dir)
   end
   
   def Content::contentFile(file)
         
      #test si le fichier existe, si non alors renvois une erreur
      if !File::exist?(file)
         return ContentError::new(404)
      end
   
      #test si c'est un contenu dynamique 
      /^.*\.(.+)$/ =~ file
      if DYNAMIC_FILES.include?($1)
         return ContentDynamic::new(file)
      else
         return ContentFile::new(file)
      end         
   end
   
end

class ContentCommon
   def put(con, state_code=200)
      con.write(
         "HTTP/1.0 #{state_code.to_s} #{STATE_CODE[state_code]}" + $crlf +
         'Server: ' + SERVER_TAG + $crlf
      )
      con.write('Connection: Keep-Alive' + $crlf) if @continue
      con.write('Content-Length: ' + self.size.to_s + $crlf)             
      con.write('Content-Type: ' + self.typeMIME + $crlf)
   end
   
   def typeMIME
      TYPE_MIME[MIME_DEFAULT]
   end   
   
   def size
      0
   end
   
   def continue?
      return @continue
   end
private
   @continue = true
end

class ContentRedirection < ContentCommon
   @file      
   def initialize(file)
      @file = file
   end
   
   def put(con)      
      super(con, 301)
      con.write("Location: " + @file[ROOT_DIR.size+1..-1] + $crlf)
      con.write($crlf)
   end
end

class ContentFile < ContentCommon
   @file
   def initialize(file)
      @file = file
   end
   
   def size
      return File::size(@file)   
   end
   
   def typeMIME
      /^.*\.(.+)$/ =~ @file #capture l'extension
      if TYPE_MIME.key?($1) then TYPE_MIME[$1] else TYPE_MIME[MIME_DEFAULT] end
   end
   
   def put(con)
      super
      con.write($crlf)
      
      #si la taille du fichier exde 1Mo alors envois par block de BIN_FILE_BUFFER Mo 
      if self.size > 1_000_000
         file = File::new(@file)
         begin
            while true
               con.write(file.read(BIN_FILE_BUFFER))
            end
         rescue
         end
      else
         IO::foreach(@file){|l|
            con.write(l)
         }
      end   
   end
end

class ContentDir < ContentCommon
   @content
   def initialize(dir)
      list = DirList::new(dir)
      @content = SERVER_PAGE.sub('{content}', list.print)
   end
   
   def size
      @content.size
   end
   
   def put(con)
      super
      con.write($crlf)
      
      con.write(@content)
   end
end

class ContentDynamic < ContentCommon
   @filename
   @size = 0
   def initialize(filename)
      @filename = filename
   end
   
   def size
      return @size   
   end
  
   def put(con)
      file_content = "$SAFE = 2;\n" + File::new(@filename).read()
      #$stdout = con
      f = File::new('tmp', 'w')
      $stdout = f
     
         t = Thread::new{
            begin
               eval(file_content)
            rescue  Exception => e
               puts e.message
               #puts '<br />'
               #e.backtrace.each{|trace|
               #   puts trace
               #   puts '<br />'
               #}               
            end      
         }.join
      f.close
      result_content = File::new('tmp', 'r').read
      $stdout = STDOUT
      @size = result_content.size
   
      super
      if $SESSION.createCookie?
         con.write("Set-Cookie: #{COOKIE_SESSION_NAME}=" + $SESSION.getID + $crlf);
      end
      con.write($crlf)
      con.write(result_content)
   end
end

class ContentError < ContentCommon
   @error_code
   @size = 0
   def initialize(error_code)
      @error_code = error_code
   end
   
   def size
      return @size   
   end
   
   def put(con)
      content = $crlf + SERVER_PAGE.sub('{content}', @error_code.to_s + ' ' + STATE_CODE[@error_code])
      @size = content.size
      
      super(con, @error_code)
      
      con.write($crlf)
      con.write(content)
   end
end
