# Construction de formulaire
# date : 8.05.05
# auteur : G.Burri

#classe permettant la validation (interface)
class Validation
   def initialize(session = nil)
      @session = session
      @erreur = nil
      @message = nil
   end
   
   def estValide(form)
      return true
   end
   
   #ret [String] : l'erreur (nil si pas d'erreur)
   def getErreur
      return @erreur
   end
   
   def getMessage
      return @message
   end
   
protected
   def setMessage(m)
      @message = m
   end
   
   def setErreur(e)
      @erreur = e
   end
end

class Formulaire
private
   # nom [String] : le nom du formulaire
   # page [String] : la page vers laquelle est redirig l'utilisateur aprs l'envoie du formulaire
   def initialize(nom = nil, pageOk = '')
      @items = Array::new #les items sont indexs par leur nom
      @items << HiddenVal::new('redirect', 'true')
      @items << HiddenVal::new('action', nom)
      @possedeColonneNom = false #si oui alors la premire colonne reprsente les noms de chaque champs
      @pageOk = pageOk
      
      @erreur = false #un des champs du formulaire ou le formulaire en lui meme n'est pas valide
      @erreurMessage = '' #message d'erreur (si une erreur s'est produite evidement)
      
      @message = false
      @messageTexte = ''
      
      @multipartData = false
      
      @validation = Validation::new #un objet Validation (classe enfant) permettant la validation du formulaire
   end
   
public
   # cre un formulaire.
   # nom [String] : le nom du formulaire
   # pageOk [String] : si tous les champs du formulaire sont valides alors redirige l'utilisateur sur cette page
   #                   sinon la page du formulaire est r-affiche
   # session [Session] : la session courante
   # ret [Formulaire] : le formulaire cr
   def Formulaire::newFormulaire(nom, pageOk, session)
      if nil == form = session.get(nom)
         session.set(nom, form = Formulaire::new(nom, pageOk))
         yield(form)
      end
      return form
   end
   
   # renvoie un formulaire mmoris dans la session courante
   def Formulaire::getFormulaire(nom, session)
      return session.get(nom)
   end
   
   # traite les entres de l'utilisateur.
   def Formulaire::input(input, session)
      #si le formulaire existe dans la session
      if nil != form = session.get(input.get('action'))
         form.input(input)         
         #si pas d'erreur alors redirige vers la page ok
         if !form.erreur?
            session.set('p', form.getPageOk)
         end
      end
   end

   # traite les entres de l'utilisateur
   def input(input)
      #mmorise chaque champs
      @items.each{|item|
         @erreur = item.setValeur(input.get(item.getNom))
         item.setAttributs(input.getAttributs(item.getNom))
      }
      #validation du formulaire
      if @erreur = !@validation.estValide(self)
         @erreurMessage = @validation.getErreur
      elsif nil != mess = @validation.getMessage
         @message = true
         @messageTexte = mess
      end
   end

   def erreur?
      return @erreur
   end
   
   def erreurMessage
      return @erreurMessage
   end

   def valide?
      return !@erreur
   end   
 
   #dfinie un code de validation pour le formulaire
   def addValidation(validation)
      @validation = validation
   end

   #retourne l'item en fonction de son nom
   def getChamp(nom)
      @items.each{|i|
         return i if i.getNom == nom
      }
   end

   def addItem(item)
      @possedeColonneNom = true if item.getNomAffiche != nil and item.getNomAffiche != ''
      @multipartData = true if item.class == DataWidget
      @items << item
   end
   
   def getPageOk
      @pageOk
   end
      
   def getHtml(input, template)
      html = '<form method="post" action="main.rb?p=' + input.get('p') +'" ' + (@multipartData ? 'enctype="multipart/form-data"' : '') + '  ><table>'
      message = false
      if @erreur
         @items << CustomWidget::new('', EkErreur::print(@erreurMessage, template))
         @erreur = false
         message = true
      elsif @message
         @items << CustomWidget::new('', EkErreur::print(@messageTexte, template))
         @message = false
         message = true
      end
      
      @items.each {|i|
         html += i.html(@possedeColonneNom)
      }
      html += '</table></form>'
      if message
         @items.pop
      end
      html
   end

end

##########################################################################

class FormItem
   def initialize(nomAffiche, nom)
      @nomAffiche = nomAffiche #le nom affich  gauche du champs
      @nom = nom #l'identifiant du formulaire
      @valeur #la valeur du champs, gal  nil lorsque le formulaire est cr
      
      #d'eventuels attributs supplmentaire sous la forme {nom => valeur}
      #par exemple le nom d'un fichier dans le cas d'un DataWidget
      @attributs
   end
   
   def getNomAffiche
      return '' if @nomAffiche == nil
      return @nomAffiche
   end
   
   # Nom de l'attribut.
   def getNom
      return @nom
   end
   
   # Dfinie la valeur de l'item
   def setValeur(val) 
      @valeur = val
   end
   
   # Dfinie les attributs sup
   def setAttributs(a)
      @attributs = a
   end
   
   # Retourne la valeur de l'item
   def getValeur
      return @valeur
   end
   
   # Retourne la valeur en escapant les '
   def getValeurEsc
      return Format::escapeBD(getValeur)
   end

   # Reinitialise la valeur
   def reinit
      @valeur = nil
   end
   
   def html(printColonneNom)
      html = '<tr>'
      html += '<td>' + getNomAffiche + '</td>' if printColonneNom
      html += '<td>' + print + '</td>'
      html += '</tr>'
      html
   end
   
protected
   
   #mthode virtuelle
   def print
      'Erreur : impossible d\affiche un composant de formulaire'
   end
end

class LineEdit < FormItem
   def initialize(nomAffiche, nom, taille, tailleMax=nil, password=false)
      super(nomAffiche, nom)
      @taille = taille
      @tailleMax = tailleMax
      @password = password
      @valeur = ''
   end
   
   # Dfinie la valeur de l'item
   def setValeur(val)
      val = '' if val == nil
      val.strip!
      super(val)
   end

protected

   def print
      '<input type="' + ( @password ? 'password' : 'text' ) + '" size="'+@taille.to_s+'" ' + (@tailleMax != nil ? 'maxLength="'+@tailleMax.to_s+'"' : '') +' name="'+@nom+'" value="'+ (!@password && getValeur != nil ? getValeur : '') + '" />'
   end
end


class TextEdit < FormItem
   def initialize(nomAffiche, nom, nbCol, nbLigne)
      super(nomAffiche, nom)
      @nbCol = nbCol
      @nbLigne = nbLigne
      @valeur = ''
   end

   # Dfinie la valeur de l'item
   def setValeur(val)
      val = '' if val == nil
      val.strip!
      super(val)
   end   
   
   def html(printColonneNom)
      html = '<tr>'
      html += '<td style="vertical-align: top" >' + getNomAffiche + '</td>' if printColonneNom
      html += '<td>' + print + '</td>'
      html += '</tr>'
      html
   end

protected
   
   def print
      '<textarea name="'+@nom+'" rows="'+@nbLigne.to_s+'" cols="'+@nbCol.to_s+'">'+ (getValeur != nil ? getValeur : '') +'</textarea>'
   end
end

class DataWidget < FormItem
   def initialize(nomAffiche, nom)
      super(nomAffiche, nom)
   end
   
   def getFilename
      return @attributs != nil ? @attributs['filename'] : nil
   end
   
   def getFilenameEsc
      return getFilename != nil ? Format::escapeBD(getFilename) : nil
   end

protected

   def print
      '<input type="file" name="'+@nom+'" />'
   end
end

class HiddenVal < FormItem
   def initialize(nom, val='')
      super('', nom)
      @val = val
   end

protected
   
   def print
      '<input type="hidden" name="'+@nom+'" value="'+@val+'" />'
   end
end

class Submit < FormItem
   def initialize(nom)
      super('', nom)
   end

protected
   
   def print
      '<input type="submit" value="'+@nom+'" />'
   end
end

class TitleWidget < FormItem
   def initialize(nomAffiche)
      super(nomAffiche, '')
   end
   
   def html(printColonneNom)
      '<tr><td colspan="2" ><h3>' + getNomAffiche + '</h3></td></tr>'
   end
end

class CustomWidget < FormItem
   def initialize(nomAffiche, html, nom='')
      super(nomAffiche, nom)
      @html = html
   end

protected
  
   def print
      @html
   end
end
