#!/usr/bin/ruby

# EliseWebServer
# Fichier : InHttpMessage.rb
# version : 1
# Date cration : 3.02.05
# Date modification : 3.02.05
#
# Analyse un message HTTP provenant du client.

require 'src/Input.rb'

class InHttpMessage
   def initialize(client_data, con)
      @input = Input::new
            
      #extrait la mthode (post, get), les options et le corps du message
      methode, @options, body = InHttpMessage::parseHTTPMess(client_data)
      
      #mmorise le niveau du referer :
      #par exemple http://site/niveau1/niveau2 possde 2 niveaux
      if getOption('Referer') != nil
         @nbLevelReferer = getOption('Referer').count('/') - 3
      end
            
      #si les templates sont activs alors cre directement une session pour pouvoir y stocker
      #le template courant
      if TEMPLATES
         @session = Session::new(getOption('Cookie'))
         @session.set('EliseCurrentTemplate', TEMPLATE_DEFAULT_DIRECTORY) if @session.get('EliseCurrentTemplate') == nil
      else
         @session = nil
      end
      @templateDir = false #si vrai alors va chercher ressource dans le template courant
      
      #change de root dir en fonction de VIRTUAL_SERVER
      @root_dir = ROOT_DIR if nil == @root_dir = VIRTUAL_SERVERS[@options['Host']]
      
      #le root dir n'a pas de '/'  la fin
      @root_dir.slice!(-1..-1) if @root_dir[-1..-1] == '/'    
            
      #si de type 'POST' alors on s'assure d'avoir lut toute les donnes
      if methode[0] == 'POST'
         length = @options['Content-Length'].to_i
         while body.size != length
            return if IO::select([con], nil, nil, MAX_CONNEXION_WAIT) == nil
            tmp = con.recv(4096)
            body += tmp #[0]
         end
      end

      #### EXTRACTION DES PARAMETRES GET & POST
      #extrait les paramtres donnes sous la forme : url?a=1&b=2

      tmp = methode[1].split('?')
      methode[1] = tmp[0]
      tmp[1].to_s.split('&').each{|var|
         tmp = var.split('=')
         @input.set(tmp[0], tmp[1])
      }

      #extrait les paramtres passs par POST
      if methode[0] == 'POST'

         #suivant le type d'envoie
         contentType = getOption('Content-Type').split('; ')
         if contentType[0] == 'multipart/form-data'
            #extrait chaque donnes du formulaire multipartie
            boundary = '--' + contentType[1][9..-1]
            body = body.split(boundary)[1..-2]
            body.each{|subData|
               subOptions, subBody = InHttpMessage::parseHTTPMess(subData, false)
               #vire le \r\n de la fin du body
               subBody.slice!(-2..-1)
               /name="(\S+)"/ =~ subOptions['Content-Disposition']
               name = $1
               @input.set(name, subBody)
               /filename="(\S+)"/ =~ subOptions['Content-Disposition']
               @input.add(name, 'filename', $1) if $1 != nil
            }
         else #application/x-www-form-urlencoded
            body.to_s.split('&').each{|var|
               tmp = var.split('=')
               replaceCodeChar(tmp[1]) #dcode les caracteres '%20'
               @input.set(tmp[0], tmp[1])
            }
         end
      end
      #### FIN EXTRACTION DES PARAMETRES GET & POST

      @filename = methode[1]

      #### TRAITEMENT DE L'URI ####
      #vire les rpetitions de '/'
      @filename.gsub!(/\/+/, '/')
      
      #remplace les caractres cods sous la forme %<hex ascii (US) code>
      #par exemple : '%20' -> ' '
      replaceCodeChar(@filename)
      
      @filename.gsub!('/..', '') #vire la remonter de rep
      #### TRAITEMENT DE L'URI ####
      
   end

   #renvoie la valeur de l'option demande, nil si elle n'existe pas
   def getOption(optionName)
      @options[optionName]
   end
   
   def eachOptions
      @options.each{|name, value|
         yield(name, value)
      }
   end

   #Renvoie le chemin absolue de la ressource demande
   def getAbsoluteFilename
      @root_dir + getFilename
   end

   def setAbsoluteFilename(filename)
      @filename = filename[@root_dir.size..-1]
   end
      
   def setRelativeFilename(filename)
      @filename = filename
   end

   #renvoie le chemin relatif de la ressource demande
   def getRelativeFilename
      return getFilename
   end
   
   def getAbsoluteDirectory
      if nil == dir = self.getAbsoluteFilename.split(/\/[^\/]+$/)[0]
         '/'
      else
         dir + (dir[-1..-1] != '/' ? '/' : '')
      end
   end
      
   def getRelativeDirectory
      if nil == dir = self.getRelativeFilename.split(/\/[^\/]+$/)[0]
         '/'
      else
         dir + (dir[-1..-1] != '/' ? '/' : '')
      end
   end
   
   #renvoie le nom du fichier
   def getFile
      /([^\/]+)$/ =~ @filename
      $1
   end
   
   #regarde si la ressource existe, si les template sont activs alors va galement regarder dans le rep
   #du template courante (session[EliseCurrentTemplate])
   def ressourceExiste?
      if !File::exist?(self.getAbsoluteFilename)         
         #p @root_dir + getFilename(true)
         if TEMPLATES and File::exist?(@root_dir + getFilename(true))
            @templateDir = true
            return true
         else
            return false
         end
      else
         return true
      end
   end

   #renvoie les donnes pass par get ou post (les deux sont mlanges)
   def getInput
      @input
   end
   
   #renvoie la session
   def getSession
      @session = Session::new(self.getOption('Cookie')) if @session == nil         
   end

private
   def getFilename(forceUseTemplate = false)
      if TEMPLATES and @nbLevelReferer != nil and (@templateDir or forceUseTemplate) 
         tab = @filename.split('/')
         tab = tab[0..@nbLevelReferer] + [@session.get('EliseCurrentTemplate')] + tab[@nbLevelReferer+1..-1]
         return tab.join('/')
      end
      return @filename
   end
   
   def replaceCodeChar(str)
      return if str == nil
      str.gsub!('+', ' ')
      str.gsub!(/%[0-9A-F]{2}/){|s|
         val = s[-2..-1].hex
         if (1..126).include?(val)
            '' << val
         end
      }
   end
   
public
   
   def InHttpMessage::parseHTTPMess(mess, hasMethode = true)
      #spare les diffrents ligne de la requte'
      options = Hash::new

      mess = mess.split("\r\n\r\n", 2)
      body = mess[1] == nil ? '' : mess[1]
      head = mess[0].split("\r\n")

      #methode contient : [<la mthode (get, post)>, <le fichier/repertoire demand>, <la version du protocole HTTP>]
      methode = head[0].split(/\s+/) if hasMethode

      #construit un hash de type : { <option> => <valeur> }
      #si le message est de type 'POST' alors ne lit pas les donnes (-1)
      head[(hasMethode ? 1 : 0)..-1].each{|line|
         line = line.split(":", 2).map!{|l| l.strip } #enlve les espace de dbut et de fin de l'option (nom et valeur)
         options[line[0]] = line[1] if line[0] != nil
      }
      hasMethode ? [methode, options, body] :  [options, body]
   end   
end
