#!/usr/bin/ruby

# EliseWebServer
# Fichier : Session.rb
# version : 1
# Date cration : 2.02.05
#
# Gestion des sessions.

require 'src/constants.rb'
require 'src/HTTPTime.rb'
require 'src/Template.rb'
require 'src/Formulaire.rb'

class Session
   @@mutex = Mutex::new
   def initialize(cookie)

      #extraction de l'id de session
      if cookie != nil
         cookie =~ /#{COOKIE_SESSION_NAME}=([^;]*)/
      end

      @clientID = $1
      @timeToLive = nil
      @createCookie = false
      @timeTolive = nil
   end
   
   #dfinit le temps de la session [s], par dfaut = infini
   def setTimeToLive(t)
      @timeTolive = t;
   end

   def get(var)
      if @clientID != nil and @@data.has_key?(@clientID)
         begin
            @@mutex.synchronize{
               return @@data[@clientID][0][var]
            }
         rescue
            return nil
         end
      end
      return nil
   end
 
   #si value == nil alors vire la variable
   def set(var, value)

      if @clientID == nil || ! @@data.has_key?(@clientID)
         @clientID = rand(1_000_000_000_000_000_000).to_s
         @@data[@clientID] = Array::new
         @@data[@clientID][0] = Hash::new
         @createCookie = true
      end
      @@mutex.synchronize{
         if value == nil
            @@data[@clientID][0].delete(var)
         else
            @@data[@clientID][0][var] = value
         end
      }
   end

   def destroy!
      @clientID = 0
      @@data.delete(@clientID)
      @createCookie = true
      @destroy = true
   end

   def getID
      @clientID
   end

   def createCookie?
      @createCookie
   end

   def getDateExpire
      return nil if @timeTolive == nil
      HTTPTime::fromTime(Time::new + @timeTolive).getHTTPcookie
   end

   def Session::storeSession
      puts '[Session]Sauvegarde des sessions..'
      f = File::new(@@DIRECTORY_SESSION, 'w')
      f.write(Marshal::dump(@@data))
      f.close
   end

private
   
   def Session::loadSession
      begin
         f = File::new(@@DIRECTORY_SESSION, 'r')
         @@data = Marshal::load(f.read)
         f.close
      rescue => e
         puts e.message
         e.backtrace.each{|f|
             puts f
         }
         puts '[Session]Erreur de chargement des sessions'
         Session::storeSession
      end
   end

   #{ <client id> => [ { <var> => <data (text)> }, <expiration time>, <last access time> ] }
   #<expiration time> est la date d'expiration de la session
   #<last access time> est la date du dernier accs de la session
   #  -> si une session n'est plus acced durant un certain temps elles automatiquement supprime
   @@data = Hash::new

   @@DIRECTORY_SESSION = Dir::pwd + '/sessions/sessions'

   @@CheckInterval = 15 * 60 #15 minutes

   #thread s'occupant de vider les sessions trop vieilles et
   #d'crire sur le disque les sessions pour les rendre persistantes
   Thread::new {
      while true
         sleep @@CheckInterval
         @@mutex.synchronize{
            begin
               Session::storeSession
            rescue Exception => e
               puts e.message
               e.backtrace.each{|f|
                   puts f
               }
            end
         }
      end
   }
end
=begin
ObjectSpace.define_finalizer(Session){|id|
   begin
      Session::storeSession            
   rescue Exception => e     
      puts e.message
     
      e.backtrace.each{|f|
         puts f
      }
   end
}
=end

