/**
  * Script de cration de la base de donnes Euphorik.
  * SGBD : PostgreSQL
  * Auteur : G.Burri
  * Date : 6.04.05
  */

--##########################################--
CREATE TABLE droit (
  id integer NOT NULL,
   
  ajout_topic boolean NOT NULL DEFAULT TRUE,
  modification_topic boolean NOT NULL DEFAULT FALSE,
  verouiller_topic boolean NOT NULL DEFAULT FALSE,
  suppression_topic boolean NOT NULL DEFAULT FALSE,
   
  ajout_news boolean NOT NULL DEFAULT FALSE,
  modification_news boolean NOT NULL DEFAULT FALSE,
  suppression_news boolean NOT NULL DEFAULT FALSE,
  proposer_news boolean NOT NULL DEFAULT TRUE,
   
  ajout_article boolean NOT NULL DEFAULT FALSE,
  modification_article boolean NOT NULL DEFAULT FALSE,
  suppression_article boolean NOT NULL DEFAULT FALSE,
   
  modification_inscrit boolean NOT NULL DEFAULT FALSE,
  -- le flag 'inscrit.ferme' est mis  TRUE (l'inscrit n'est pas reelment supprim)
  suppression_inscrit boolean NOT NULL DEFAULT FALSE,

  -- peut modifier le status d'une personne ayant un statut.niveau <= modification_statut_inscrit
  -- le status modifi doit tre >= 0 et <= modification_statut_inscrit
  modification_statut_inscrit integer NOT NULL DEFAULT 0,
   
  ajout_theme boolean NOT NULL DEFAULT FALSE,
  modification_theme boolean NOT NULL DEFAULT FALSE,
  suppression_theme boolean NOT NULL DEFAULT FALSE,

  PRIMARY KEY(id)
);
--cree un certains nombre de droits prdfinis
--Inscrit simple :
INSERT INTO droit
(id,
ajout_topic,         modification_topic,   verouiller_topic,           suppression_topic,
ajout_news,           modification_news,    suppression_news,           proposer_news,
ajout_article,        modification_article, suppression_article,
modification_inscrit, suppression_inscrit,  modification_statut_inscrit,
ajout_theme,          modification_theme,   suppression_theme)
VALUES
(1,
TRUE,  FALSE, FALSE, FALSE,
FALSE, FALSE, FALSE, TRUE,
FALSE, FALSE, FALSE,
FALSE, FALSE, 0,
FALSE, FALSE, FALSE);
--Membre :
INSERT INTO droit
(id, 
ajout_topic,          modification_topic,   verouiller_topic,           suppression_topic,
ajout_news,           modification_news,    suppression_news,           proposer_news,
ajout_article,        modification_article, suppression_article,
modification_inscrit, suppression_inscrit,  modification_statut_inscrit,
ajout_theme,          modification_theme,   suppression_theme)
VALUES
(2,
TRUE,  FALSE, FALSE, FALSE,
TRUE,  TRUE,  FALSE, FALSE,
TRUE,  TRUE,  FALSE,
FALSE, FALSE, 0,
TRUE,  FALSE, FALSE);
--Modo :
INSERT INTO droit
(id, 
ajout_topic,          modification_topic,   verouiller_topic,           suppression_topic,
ajout_news,           modification_news,    suppression_news,           proposer_news,
ajout_article,        modification_article, suppression_article,
modification_inscrit, suppression_inscrit,  modification_statut_inscrit,
ajout_theme,          modification_theme,   suppression_theme)
VALUES
(3,
TRUE,  TRUE, TRUE, TRUE,
TRUE,  TRUE, TRUE, FALSE,
TRUE,  TRUE, TRUE,
FALSE, FALSE, 10,
TRUE,  TRUE, TRUE);
--Root :
INSERT INTO droit
(id,
ajout_topic,          modification_topic,   verouiller_topic,           suppression_topic,
ajout_news,           modification_news,    suppression_news,           proposer_news,
ajout_article,        modification_article, suppression_article,
modification_inscrit, suppression_inscrit,  modification_statut_inscrit,
ajout_theme,          modification_theme,   suppression_theme)
VALUES
(4,
TRUE,  TRUE, TRUE, TRUE,
TRUE,  TRUE, TRUE, FALSE,
TRUE,  TRUE, TRUE,
TRUE,  TRUE, 30,
TRUE,  TRUE, TRUE);

--##########################################--
CREATE TABLE statut (
  id integer NOT NULL,
  droit integer NOT NULL REFERENCES droit,
  
  niveau integer NOT NULL DEFAULT 0,
  nom varchar(20) NOT NULL,

  PRIMARY KEY(id)
);
CREATE INDEX statut_droit_INDEX on statut(droit);
--cree un certains nombre de statuts prdfinis
--Inscrit simple :
INSERT INTO statut (id, droit, niveau, nom) VALUES (1, 1, 0,  'inscrit');
--Membre
INSERT INTO statut (id, droit, niveau, nom) VALUES (2, 2, 10, 'membre');
--Modo
INSERT INTO statut (id, droit, niveau, nom) VALUES (3, 3, 20, 'modo');
--Root
INSERT INTO statut (id, droit, niveau, nom) VALUES (4, 4, 30, 'root');

--##########################################--
CREATE TABLE grade (
  id integer NOT NULL,
 
  nom varchar(20) NOT NULL,
  niveau integer DEFAULT 1,

  PRIMARY KEY(id)
);
--cree un certains nombre de grades prdfinis
INSERT INTO grade (id, nom, niveau) VALUES (1, 'initi', 10);
INSERT INTO grade (id, nom, niveau) VALUES (2, 'membre', 20);
INSERT INTO grade (id, nom, niveau) VALUES (3, 'vtran', 40);
INSERT INTO grade (id, nom, niveau) VALUES (4, 'officier', 50);
INSERT INTO grade (id, nom, niveau) VALUES (5, 'leader', 60);

--##########################################--
CREATE TABLE inscrit (
  id serial NOT NULL,
  grade integer NOT NULL REFERENCES grade DEFAULT 1,
  statut integer NOT NULL REFERENCES statut DEFAULT 1,

  pseudo varchar(30) UNIQUE NOT NULL,
  pass varchar(50) NOT NULL,

  mail varchar(50) NOT NULL DEFAULT '',
  date_creation timestamp NOT NULL,

  -- aprs l'inscription au site validation est  FALSE tant que la personne
  -- n'a pas confirm son inscription (suite  la reception d'un mail)
  -- un thread s'occuper de supprimer les personnes qui n'auraient confirm aprs un certain dlais
  validation boolean NOT NULL DEFAULT FALSE,
  -- permet de valider un compte
  validation_jeton varchar(200) NOT NULL DEFAULT '',

  icq varchar(40) NOT NULL DEFAULT '',
  msn varchar(40) NOT NULL DEFAULT '',
  jabber varchar(40) NOT NULL DEFAULT '',
  site_perso varchar(40) NOT NULL DEFAULT '',
  -- apparait dans le forum (topic)
  citation varchar(200) NOT NULL DEFAULT '',
  -- apparait galement dans le forum (topic)
  signature varchar(200) NOT NULL DEFAULT '',
  histoire_personnage text NOT NULL DEFAULT '',
  pensee_philo text NOT NULL DEFAULT '',
  -- le numro de l'avatar parmis ceux proposs
  avatar_num integer NULL,
  avatar_image bytea NULL,
  nom varchar(40) NOT NULL DEFAULT '',
  prenom varchar(40) NOT NULL DEFAULT '',
  domicile varchar(200) NOT NULL DEFAULT '',
  lien_personnage varchar(100) NOT NULL DEFAULT '',
  ferme bool NOT NULL DEFAULT FALSE, --un compte peut tre ferm par un admin

  PRIMARY KEY(id)
);
CREATE INDEX inscrit_grade_INDEX on inscrit(grade);
CREATE INDEX inscrit_statut_INDEX on inscrit(statut);
-- cration automatique de l'utilisateur root
INSERT INTO inscrit
(id, grade, statut, pseudo,                               pass, date_creation, validation) VALUES
( 1,     5,      4, 'root', '57ae9ee5592123dd79c8912bf768cf99',         now(),       true);

--##########################################--
CREATE TABLE groupe_commentaires (
  id serial NOT NULL,
  PRIMARY KEY(id)
);

--##########################################--
CREATE TABLE commentaire (
  id serial NOT NULL,
  commentaire_ref integer NULL REFERENCES commentaire,
  groupe_commentaires integer NOT NULL REFERENCES groupe_commentaires ON DELETE CASCADE,
  inscrit integer NOT NULL REFERENCES inscrit,
  
  date timestamp NOT NULL,
  commentaire text NOT NULL, 

  PRIMARY KEY(id)
);
CREATE INDEX commentaire_commentaire_ref_INDEX on commentaire(commentaire_ref);
CREATE INDEX commentaire_groupe_commentaires_INDEX on commentaire(groupe_commentaires);
CREATE INDEX commentaire_inscrit_INDEX on commentaire(inscrit);

--##########################################--
CREATE TABLE evaluation (
  id serial NOT NULL,
  inscrit integer NOT NULL REFERENCES inscrit,
  commentaire integer NOT NULL REFERENCES commentaire ON DELETE CASCADE,

  evaluation integer NOT NULL,

  PRIMARY KEY(id)
);
CREATE INDEX evaluation_commentaire_INDEX on evaluation(commentaire);
CREATE INDEX evaluation_inscrit_INDEX on evaluation(inscrit);

--##########################################--
CREATE TABLE mp (
  id serial NOT NULL,
  expediteur integer NOT NULL REFERENCES inscrit,
  destinataire integer NOT NULL REFERENCES inscrit,
  mp integer NOT NULL REFERENCES mp, -- en reponse  mp

  date timestamp NOT NULL,
  lu boolean NOT NULL DEFAULT FALSE,
  message text NOT NULL,

  PRIMARY KEY(id)
);
CREATE INDEX mp_expediteur_INDEX on mp(expediteur);
CREATE INDEX mp_destinataire_INDEX on mp(destinataire);
CREATE INDEX mp_mp_INDEX on mp(mp);

--##########################################--
CREATE TABLE tribune (
  id serial NOT NULL,
  inscrit integer NOT NULL REFERENCES inscrit,
  
  date timestamp NOT NULL,
  message varchar(250) NOT NULL,

  PRIMARY KEY(id)
);
CREATE INDEX tribune_inscrit_INDEX on tribune(inscrit);

--##########################################--
CREATE TABLE lu (
  id serial NOT NULL,
  groupe_commentaires integer NOT NULL REFERENCES groupe_commentaires,
  inscrit integer NOT NULL REFERENCES inscrit,

  -- date de la dernire lecture, les messages posts aprs cette date sont considr comme non-lu
  date timestamp NOT NULL, 
 
  PRIMARY KEY(id)
);
CREATE INDEX lu_groupe_commentaires_INDEX on lu(groupe_commentaires);
CREATE INDEX lu_inscrit_INDEX on lu(inscrit);

--##########################################--
CREATE TABLE groupe_images (
  id serial NOT NULL,

  PRIMARY KEY(id)
);

--##########################################--
CREATE TABLE image (
  id serial NOT NULL,
  groupe_images integer NOT NULL REFERENCES groupe_images,
  -- une image peut avoir des commentaires associs
  groupe_commentaires integer NOT NULL REFERENCES groupe_commentaires,
  
  nom_image varchar(250) NOT NULL DEFAULT '', --le nom de l'image
  image bytea NOT NULL, -- l'image sans redimensionnement
  image_redim bytea NULL, --l'image redimenssionne
  numero integer NOT NULL, -- un numero permettant le rfrencement de l'image dans une information
  legende varchar(200) NOT NULL DEFAULT '',
 
  PRIMARY KEY(id)
);
CREATE INDEX image_groupe_images_INDEX on image(groupe_images);
CREATE INDEX image_groupe_commentaires_INDEX on image(groupe_commentaires);

--##########################################--
CREATE TABLE galerie (
  id serial NOT NULL,
  groupe_images integer NOT NULL REFERENCES groupe_images,

  titre varchar(100),

  PRIMARY KEY(id)
);
CREATE INDEX galerie_groupe_images on galerie(groupe_images);

--##########################################--
CREATE TABLE theme (
  id serial NOT NULL,
  inscrit integer NOT NULL REFERENCES inscrit,
  
  nom varchar(40) NOT NULL,  

  PRIMARY KEY(id)
);
CREATE INDEX theme_inscrit_INDEX on theme(inscrit);
-- ajout de thmes prdfinis
INSERT INTO theme (inscrit, nom) VALUES (1, 'World of Warcraft');
INSERT INTO theme (inscrit, nom) VALUES (1, 'Informatique');
INSERT INTO theme (inscrit, nom) VALUES (1, 'Jeux vido');
INSERT INTO theme (inscrit, nom) VALUES (1, 'LAN');
INSERT INTO theme (inscrit, nom) VALUES (1, 'Dbilit');

--##########################################--
CREATE TABLE historique_contenu (
  id serial NOT NULL,

  PRIMARY KEY(id)
);

--##########################################--
CREATE TABLE contenu (
  id serial NOT NULL,
  inscrit integer NOT NULL REFERENCES inscrit,
  historique_contenu integer NOT NULL REFERENCES historique_contenu,
  
  date timestamp NOT NULL,
  contenu text NOT NULL,
  commentaire_modification varchar(200) NOT NULL DEFAULT '',
  
  PRIMARY KEY(id)
);
CREATE INDEX contenu_inscrit_INDEX on contenu(inscrit);
CREATE INDEX contenu_historique_contenu_INDEX on contenu(historique_contenu);

--##########################################--
CREATE TABLE information (
  id serial NOT NULL,
  -- les contenu de l'information, le plus rcent tant le contenu de base
  historique_contenu integer NOT NULL REFERENCES historique_contenu,
  groupe_images integer NOT NULL REFERENCES groupe_images, -- un groupe d'images, peut-tre vide
  groupe_commentaires integer NOT NULL REFERENCES groupe_commentaires, -- les commentaires, le groupe peut tre vide
  theme integer NOT NULL REFERENCES theme,
  -- la personne ayant poste l'information, doit correspondre au premier contenu.inscrit
  inscrit integer NOT NULL REFERENCES inscrit,
  
  titre varchar(100) NOT NULL,
  date timestamp NOT NULL,
  -- les personnes ayant un status < que level visualisation ne peuvent pas voir l'information
  level_visualisation integer NOT NULL DEFAULT 0,
  
  PRIMARY KEY(id)
);
CREATE INDEX information_historique_contenu_INDEX on information(historique_contenu);
CREATE INDEX information_groupe_images_INDEX on information(groupe_images);
CREATE INDEX information_groupe_commentaires_INDEX on information(groupe_commentaires);
CREATE INDEX information_theme_INDEX on information(theme);
CREATE INDEX information_inscrit_INDEX on information(inscrit);

--##########################################--
CREATE TABLE evenement (
  id_evenement serial NOT NULL,

  date_evenement timestamp NOT NULL, -- la date de l'evenement

  PRIMARY KEY(id_evenement)
) INHERITS(information);

--##########################################--
CREATE TABLE news (
  id_news serial NOT NULL,

  -- permet d'interdire la modification par les autres membres...
  -- (reflechire sur l'utilit...)
  modifiable boolean NOT NULL DEFAULT TRUE,

  PRIMARY KEY(id_news)
) INHERITS(information);

--##########################################--
CREATE TABLE article (
  id_article serial NOT NULL,

  -- permet d'interdire la modification par les autres membres...
  -- (reflechire sur l'utilit...)
  modifiable boolean NOT NULL DEFAULT TRUE,
 
  PRIMARY KEY(id_article)
) INHERITS(information);

--##########################################--
CREATE TABLE sondage (
  id serial NOT NULL,

  question varchar(200) NOT NULL,

  PRIMARY KEY(id)
);

--##########################################--
CREATE TABLE topic (
  id_topic serial NOT NULL,
  sondage integer NULL REFERENCES sondage ON DELETE CASCADE,

  -- un topic verouill ne peut plus tre dit par son auteur et
  -- aucun commentaire ne peut plus tre post 
  verouille boolean NOT NULL DEFAULT FALSE,

  PRIMARY KEY(id_topic)
) INHERITS(information);
CREATE INDEX topic_sondage_INDEX on topic(sondage);

--##########################################--
-- un sondage possde plusieurs rponses
CREATE TABLE sondage_reponse (
  id serial NOT NULL,
  sondage integer NOT NULL REFERENCES sondage ON DELETE CASCADE,
  
  reponse varchar(200) NOT NULL,
  nb_vote integer NOT NULL DEFAULT 0, -- valeur redondante, eventuellement enlever

  PRIMARY KEY(id)
);
CREATE INDEX sondage_reponse_sondage_INDEX on sondage_reponse(sondage);

--##########################################--
CREATE TABLE vote (
  id serial NOT NULL,
  inscrit integer NOT NULL REFERENCES inscrit, -- le personne qui vote

  date timestamp NOT NULL, -- la date du vote

  PRIMARY KEY(id)
);
CREATE INDEX vote_inscrit_INDEX on vote(inscrit);

--##########################################--
-- personnes prsente  un certain evenement
CREATE TABLE presents ( 
  id serial NOT NULL,
  evenement integer NOT NULL REFERENCES evenement ON DELETE CASCADE,
  inscrit integer  NOT NULL REFERENCES inscrit,

  commentaire text NOT NULL DEFAULT '', -- un commentaire de l'inscrit quant  sa prsence 

  PRIMARY KEY(id)
);
CREATE INDEX presents_evenement_INDEX on presents(evenement);
CREATE INDEX presents_inscrit_INDEX on presents(inscrit);

--##########################################--
-- personnes absentes  un certain evenement
CREATE TABLE absents ( 
  id serial NOT NULL,
  evenement integer NOT NULL REFERENCES evenement ON DELETE CASCADE,
  inscrit integer NOT NULL REFERENCES inscrit,

  commentaire text NOT NULL DEFAULT '', -- un commentaire de l'inscrit quant  sa prsence

  PRIMARY KEY(id)
);
CREATE INDEX absents_evenement_INDEX on absents(evenement);
CREATE INDEX absents_inscrit_INDEX on absents(inscrit);

--##########################################--
-- rapport de bogue concernant le site de la part des inscrits
CREATE TABLE bogue (
  id serial NOT NULL,
  inscrit integer NOT NULL REFERENCES inscrit,
  
  titre varchar(100) NOT NULL,
  description text NOT NULL,
  date timestamp NOT NULL,

  PRIMARY KEY(id)
);