# Module d'accs  une base de donnes grant les multis connexions (multithread)
# date : 12.04.05
# auteur : G.Burri

#sous unix
#$LOAD_PATH << '/usr/local/lib/ruby/gems/1.8/gems/postgres-pr-0.4.0/lib'
#sous windaube
#$LOAD_PATH << 'C:\\ruby\\lib\\ruby\\gems\\1.8\\gems\\postgres-pr-0.4.0\\lib'
#require 'postgres-pr/connection'

require 'dbi' #un autre driver...
require 'thread'

# Exception leve lorsque il n'y a plus de connexiondisponible
class NoConnectionAvailableException < Exception
end

class Database
   @@NbConnexion = 1

   def Database::setNbConnexion(n)
      @@NbConnexion = n
   end

   def initialize(bd, user, pass, host = nil)
      @cons = Array::new
     
      # ouvre n connexions
      @@NbConnexion.times{
         @cons << DBI::connect('DBI:Pg:euphorikkrew', 'postgres', '')
      }
      
      @lastResults = Array::new(@@NbConnexion)
      @occupations = Array::new(@@NbConnexion)
      @occupations.fill(false, 0, @@NbConnexion)
      
      @mutex = Mutex::new
   end
   
   #retourne le numero de la requte
   #p2 : binds..
   def query(q, *blobs)
      #recherche une connexion libre
      libre = 0
      @mutex.synchronize{         
         @occupations.each{|o|
            break if o == false
            libre += 1
         }
         raise NoConnectionAvailableException if libre == @@NbConnexion
         #puts "libre = #{libre}"
         @occupations[libre] = true
      }
      
      if blobs[0] == nil or blobs.empty?
         @lastResults[libre] = @cons[libre].execute(q).fetch_all
      else
         req = @cons[libre].prepare(q)         
         req.bind_param(1, encodeBytea(blobs[0]))         
         @lastResults[libre] = req.execute
      end
      
      libre
   end
   
   #faudrait utiliser la mthode de 'Pg' ...
   def encodeBytea(str)
      # TODO: use quote function of Pg driver
      a = str.split(/\\/, -1).collect! {|s|
         s.gsub!(/'/,    "\\\\047")  # '  => \\047
         s.gsub!(/\000/, "\\\\000")  # \0 => \\000
         s
      }
      a.join("\\\\")                # \  => \\
   end

   def eachRow(num)
      #@lastResults[num].rows.each{|r|
      #   yield(r)
      #}
      @lastResults[num].each{|l|
         yield(l)
      }
      #req = @lastResults[num]
      #while ligne = req.fetch
      #   yield(ligne)
      #end
   end
   
   def rEachRow(num)
      #@lastResults[num].rows.reverse_each{|r|
      #   yield(r)
      #}
      @lastResults[num].reverse_each{|l|
         yield(l)
      }
      
      #while ligne = req.fetch
      #   yield(ligne)
      #end
   end
   
   def firstRow(num)
      @lastResults[num][0]
   end
   
   def nbRow(num)
      @lastResults[num].size
   end
   
   def freeQuery(num)
      @mutex.synchronize{
         @occupations[num] = false
      }
   end
   
   def beginTrans()
      return query("BEGIN")
   end
   
   def commitTrans(num)
      @con[num].query("COMMIT");
   end

   def close
      @cons.each{|c|
         c.disconnect
      }
   end
end
